/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.bigbluebutton.ui;

import java.net.URI;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.olat.core.CoreSpringFactory;
import org.olat.core.gui.components.form.flexible.FormItem;
import org.olat.core.gui.components.form.flexible.elements.DateChooser;
import org.olat.core.gui.components.form.flexible.elements.SingleSelection;
import org.olat.core.gui.components.form.flexible.elements.TextElement;
import org.olat.core.gui.components.util.KeyValues;
import org.olat.core.gui.translator.Translator;
import org.olat.core.util.StringHelper;
import org.olat.modules.bigbluebutton.BigBlueButtonDispatcher;
import org.olat.modules.bigbluebutton.BigBlueButtonManager;
import org.olat.modules.bigbluebutton.BigBlueButtonMeeting;
import org.olat.modules.bigbluebutton.BigBlueButtonMeetingLayoutEnum;
import org.olat.modules.bigbluebutton.BigBlueButtonMeetingTemplate;

/**
 * 
 * Initial date: 14 Sep 2020<br>
 * @author uhensler, urs.hensler@frentix.com, http://www.frentix.com
 *
 */
public class BigBlueButtonUIHelper {
	
	public static void updateTemplateInformations(SingleSelection templateEl, FormItem externalLinkEl, List<BigBlueButtonMeetingTemplate> templates) {
		templateEl.setExampleKey(null, null);
		if(templateEl.isOneSelected()) {
			BigBlueButtonMeetingTemplate template = getSelectedTemplate(templateEl, templates);
			if(template != null && template.getMaxParticipants() != null) {
				Integer maxConcurrentInt = template.getMaxConcurrentMeetings();
				String maxConcurrent = (maxConcurrentInt == null ? " ∞" : maxConcurrentInt.toString());
				String[] args = new String[] { template.getMaxParticipants().toString(), maxConcurrent};
				if(template.getWebcamsOnlyForModerator() != null && template.getWebcamsOnlyForModerator().booleanValue()) {
					templateEl.setExampleKey("template.explain.max.participants.with.webcams.mod", args);
				} else {
					templateEl.setExampleKey("template.explain.max.participants", args);
				}
			}
			externalLinkEl.setVisible(template != null && template.isExternalUsersAllowed());
		} else {
			externalLinkEl.setVisible(false);
		}
	}
	
	public static BigBlueButtonMeetingTemplate getSelectedTemplate(SingleSelection templateEl, List<BigBlueButtonMeetingTemplate> templates) {
		if (!templateEl.isOneSelected()) return null;
		
		String selectedTemplateId = templateEl.getSelectedKey();
		return templates.stream()
					.filter(tpl -> selectedTemplateId.equals(tpl.getKey().toString()))
					.findFirst()
					.orElse(null);
	}
	
	public static boolean isWebcamLayoutAvailable(BigBlueButtonMeetingTemplate template) {
		if(template == null) {
			return true;
		}
		return template.getWebcamsOnlyForModerator() == null || !template.getWebcamsOnlyForModerator().booleanValue();
	}
	
	public static void updateLayoutSelection(SingleSelection layoutEl, Translator translator, boolean webcamAvailable) {
		if(webcamAvailable && layoutEl.getKeys().length == 1) {
			KeyValues layoutKeyValues = new KeyValues();
			layoutKeyValues.add(KeyValues.entry(BigBlueButtonMeetingLayoutEnum.standard.name(), translator.translate("layout.standard")));
			layoutKeyValues.add(KeyValues.entry(BigBlueButtonMeetingLayoutEnum.webcam.name(), translator.translate("layout.webcam")));
			layoutEl.setKeysAndValues(layoutKeyValues.keys(), layoutKeyValues.values(), null);
		} else if(!webcamAvailable && layoutEl.getKeys().length > 1) {
			layoutEl.select(BigBlueButtonMeetingLayoutEnum.standard.name(), true);
			
			KeyValues layoutKeyValues = new KeyValues();
			layoutKeyValues.add(KeyValues.entry(BigBlueButtonMeetingLayoutEnum.standard.name(), translator.translate("layout.standard")));
			layoutEl.setKeysAndValues(layoutKeyValues.keys(), layoutKeyValues.values(), null);
		}
		
		layoutEl.setVisible(layoutEl.getKeys().length > 1);
	}

	public static boolean validateReadableIdentifier(TextElement externalLinkEl, BigBlueButtonMeeting meeting) {
		boolean allOk = true;
		
		externalLinkEl.clearError();
		if(externalLinkEl.isVisible()) {
			String identifier = externalLinkEl.getValue();
			if (StringHelper.containsNonWhitespace(externalLinkEl.getValue())) {
				if(identifier.length() > 64) {
					externalLinkEl.setErrorKey("form.error.toolong", new String[] { "64" });
					allOk &= false;
				} else if(getBigBlueButtonManager().isIdentifierInUse(identifier, meeting)) {
					externalLinkEl.setErrorKey("error.identifier.in.use", null);
					allOk &= false;
				} else {
					try {
						URI uri = new URI(BigBlueButtonDispatcher.getMeetingUrl(identifier));
						uri.normalize();
					} catch(Exception e) {
						externalLinkEl.setErrorKey("error.identifier.url.not.valid", new String[] { e.getMessage() });
						allOk &= false;
					}
				}
				externalLinkEl.setExampleKey("noTransOnlyParam", new String[] {BigBlueButtonDispatcher.getMeetingUrl(identifier)});			
			} else {
				externalLinkEl.setExampleKey(null, null);
			}
		}

		return allOk;
	}
	
	public static boolean validateTime(TextElement el, long maxValue) {
		boolean allOk = true;
		el.clearError();
		if(StringHelper.containsNonWhitespace(el.getValue())) {
			if(!StringHelper.isLong(el.getValue())) {
				el.setErrorKey("form.error.nointeger", null);
				allOk &= false;
			} else if(Long.parseLong(el.getValue()) > maxValue) {
				el.setErrorKey("error.too.long.time", new String[] { Long.toString(maxValue) });
				allOk &= false;
			}
		}
		return allOk;
	}
	
	public static boolean validateDuration(DateChooser startDateEl, TextElement leadTimeEl, DateChooser endDateEl,
			TextElement followupTimeEl, BigBlueButtonMeetingTemplate template) {
		boolean allOk = true;
		
		Date start = startDateEl.getDate();
		Date end = endDateEl.getDate();
		if(template != null && template.getMaxDuration() != null && start != null && end != null) {
			// all calculation in milli-seconds
			long realStart = start.getTime() - (60 * 1000 * getLongOrZero(leadTimeEl));
			long realEnd = end.getTime() + (60 * 1000 * getLongOrZero(followupTimeEl));
			long duration = realEnd - realStart;
			long maxDuration  = (60 * 1000 * template.getMaxDuration());
			if(duration > maxDuration) {
				endDateEl.setErrorKey("error.duration", new String[] { template.getMaxDuration().toString() });
				allOk &= false;
			}
		}
		return allOk;
	}
	
	public static boolean validateSlot(DateChooser startDateEl, TextElement leadTimeEl, DateChooser endDateEl,
			TextElement followupTimeEl, BigBlueButtonMeeting meeting, BigBlueButtonMeetingTemplate template) {
		boolean allOk = true;
		
		boolean slotFree = getBigBlueButtonManager().isSlotAvailable(meeting, template,
				startDateEl.getDate(), getLongOrZero(leadTimeEl), endDateEl.getDate(), getLongOrZero(followupTimeEl));
		if(!slotFree) {
			startDateEl.setErrorKey("server.overloaded", null);
			allOk &= false;
		}
		
		return allOk;
	}
	
	public static boolean validatePermanentSlot(SingleSelection templateEl, BigBlueButtonMeeting meeting, BigBlueButtonMeetingTemplate template) {
		boolean allOk = true;
		
		Calendar cal = Calendar.getInstance();
		cal.add(Calendar.YEAR, 12);
		Date endDate = cal.getTime();
		
		boolean slotFree = getBigBlueButtonManager().isSlotAvailable(meeting, template,
				new Date(), 0, endDate, 0);
		if(!slotFree) {
			templateEl.setErrorKey("server.overloaded", null);
			allOk &= false;
		}
		
		return allOk;
	}
	
	public static long getLongOrZero(TextElement textElement) {
		long followupTime = 0;
		if(textElement.isVisible() && StringHelper.isLong(textElement.getValue())) {
			followupTime = Long.valueOf(textElement.getValue());
		}
		return followupTime;
	}
	
	private static BigBlueButtonManager getBigBlueButtonManager() {
		return CoreSpringFactory.getImpl(BigBlueButtonManager.class);
	}
	
}
